(* ::Package:: *)

(*
  AstroDevice
  ASCOM CCD Camera, filter wheel, focuser, dome Interface, telescope mount.
  Version 1.0
  T. Sherlock
*)

BeginPackage["AstroDevice`", {"NETLink`"}]

(*
  Public functions
*)

InitDeviceInterface::usage="InitDeviceInterface[] initializes the interface (camera, filter wheel, focuser)";
InitDeviceInterface32::usage="InitDevicegInterface32[] initializes the interface (camera, filter wheel, focuser) for systems with 32 bit drivers";

(* Telescope Properties *)
SetScopeConnected::usage = "SetScopeConnected[theScope, val] Set True to connect to the device hardware. Set False to disconnect from the device hardware. You can also read the property to check whether it is connected. This reports the current hardware state.";
SetScopeDeclinationRate::usage = "SetScopeDeclinationRate[theScope, val] The declination tracking rate (arcseconds per SI second, default = 0.0)";
SetScopeDoesRefraction::usage = "SetScopeDoesRefraction[theScope, val] True if the telescope or driver applies atmospheric refraction to coordinates.";
SetScopeGuideRateDeclination::usage = "SetScopeGuideRateDeclination[theScope, val] The current Declination movement rate offset for telescope guiding (degrees/sec)";
SetScopeGuideRateRightAscension::usage = "SetScopeGuideRateRightAscension[theScope, val] The current Right Ascension movement rate offset for telescope guiding (degrees/sec)";
SetScopeRightAscensionRate::usage = "SetScopeRightAscensionRate[theScope, val] The right ascension tracking rate offset from sidereal (seconds per sidereal second, default = 0.0)";
SetScopeSideOfPier::usage = "SetScopeSideOfPier[theScope, val] Indicates the pointing state of the mount.";
SetScopeSiteElevation::usage = "SetScopeSiteElevation[theScope, val] The elevation above mean sea level (meters) of the site at which the telescope is located";
SetScopeSiteLatitude::usage = "SetScopeSiteLatitude[theScope, val] The geodetic(map) latitude (degrees, positive North, WGS84) of the site at which the telescope is located.";
SetScopeSiteLongitude::usage = "SetScopeSiteLongitude[theScope, val] The longitude (degrees, positive East, WGS84) of the site at which the telescope is located.";
SetScopeSlewSettleTime::usage = "SetScopeSlewSettleTime[theScope, val] Specifies a post-slew settling time (sec.).";
SetScopeTargetDeclination::usage = "SetScopeTargetDeclination[theScope, val] The declination (degrees, positive North) for the target of an equatorial slew or sync operation";
SetScopeTargetRightAscension::usage = "SetScopeTargetRightAscension[theScope, val] The right ascension (hours) for the target of an equatorial slew or sync operation";
SetScopeTracking::usage = "SetScopeTracking[theScope, val] The state of the telescope's sidereal tracking drive.";
SetScopeTrackingRate::usage = "SetScopeTrackingRate[theScope, val] The current tracking rate of the telescope's sidereal drive";
SetScopeUTCDate::usage = "SetScopeUTCDate[theScope, val] The UTC date/time of the telescope's internal clock";

GetScopeAlignmentMode::usage = "GetScopeAlignmentMode[theScope] The alignment mode of the mount.";
GetScopeAltitude::usage = "GetScopeAltitude[theScope] The Altitude above the local horizon of the telescope's current position (degrees, positive up)";
GetScopeApertureArea::usage = "GetScopeApertureArea[theScope] The area of the telescope's aperture, taking into account any obstructions (square meters)";
GetScopeApertureDiameter::usage = "GetScopeApertureDiameter[theScope] The telescope's effective aperture diameter (meters)";
GetScopeAtHome::usage = "GetScopeAtHome[theScope] True if the telescope is stopped in the Home position. Set only following a FindHome() operation, and reset with any slew operation. This property must be False if the telescope does not support homing.";
GetScopeAtPark::usage = "GetScopeAtPark[theScope] True if the telescope has been put into the parked state by the see Park() method. Set False by calling the Unpark() method.";
GetScopeAzimuth::usage = "GetScopeAzimuth[theScope] The azimuth at the local horizon of the telescope's current position (degrees, North-referenced, positive East/clockwise).";
GetScopeCanFindHome::usage = "GetScopeCanFindHome[theScope] True if this telescope is capable of programmed finding its home position (FindHome() method).";
GetScopeCanPark::usage = "GetScopeCanPark[theScope] True if this telescope is capable of programmed parking (Park()method)";
GetScopeCanPulseGuide::usage = "GetScopeCanPulseGuide[theScope] True if this telescope is capable of software-pulsed guiding (via the PulseGuide(GuideDirections, Int32) method)";
GetScopeCanSetDeclinationRate::usage = "GetScopeCanSetDeclinationRate[theScope] True if the DeclinationRate property can be changed to provide offset tracking in the declination axis.";
GetScopeCanSetGuideRates::usage = "GetScopeCanSetGuideRates[theScope] True if the guide rate properties used for PulseGuide(GuideDirections, Int32) can be adjusted.";
GetScopeCanSetPark::usage = "GetScopeCanSetPark[theScope] True if this telescope is capable of programmed setting of its park position (SetPark() method)";
GetScopeCanSetPierSide::usage = "GetScopeCanSetPierSide[theScope] True if the SideOfPier property can be set, meaning that the mount can be forced to flip.";
GetScopeCanSetRightAscensionRate::usage = "GetScopeCanSetRightAscensionRate[theScope] True if the RightAscensionRate property can be changed to provide offset tracking in the right ascension axis.";
GetScopeCanSetTracking::usage = "GetScopeCanSetTracking[theScope] True if the Tracking property can be changed, turning telescope sidereal tracking on and off.";
GetScopeCanSlew::usage = "GetScopeCanSlew[theScope] True if this telescope is capable of programmed slewing (synchronous or asynchronous) to equatorial coordinates";
GetScopeCanSlewAltAz::usage = "GetScopeCanSlewAltAz[theScope] True if this telescope is capable of programmed slewing (synchronous or asynchronous) to local horizontal coordinates";
GetScopeCanSlewAltAzAsync::usage = "GetScopeCanSlewAltAzAsync[theScope] True if this telescope is capable of programmed asynchronous slewing to local horizontal coordinates";
GetScopeCanSlewAsync::usage = "GetScopeCanSlewAsync[theScope] True if this telescope is capable of programmed asynchronous slewing to equatorial coordinates.";
GetScopeCanSync::usage = "GetScopeCanSync[theScope] True if this telescope is capable of programmed syncing to equatorial coordinates.";
GetScopeCanSyncAltAz::usage = "GetScopeCanSyncAltAz[theScope] True if this telescope is capable of programmed syncing to local horizontal coordinates";
GetScopeCanUnpark::usage = "GetScopeCanUnpark[theScope] True if this telescope is capable of programmed unparking (Unpark() method).";
GetScopeConnected::usage = "GetScopeConnected[theScope] Set True to connect to the device hardware. Set False to disconnect from the device hardware. You can also read the property to check whether it is connected. This reports the current hardware state.";
GetScopeDeclination::usage = "GetScopeDeclination[theScope] The declination (degrees) of the telescope's current equatorial coordinates, in the coordinate system given by the EquatorialSystem property. Reading the property will raise an error if the value is unavailable.";
GetScopeDeclinationRate::usage = "GetScopeDeclinationRate[theScope] The declination tracking rate (arcseconds per SI second, default = 0.0)";
GetScopeDescription::usage = "GetScopeDescription[theScope] Returns a description of the device, such as manufacturer and model number. Any ASCII characters may be used.";
GetScopeDoesRefraction::usage = "GetScopeDoesRefraction[theScope] True if the telescope or driver applies atmospheric refraction to coordinates.";
GetScopeDriverInfo::usage = "GetScopeDriverInfo[theScope] Descriptive and version information about this ASCOM driver.";
GetScopeDriverVersion::usage = "GetScopeDriverVersion[theScope] A string containing only the major and minor version of the driver.";
GetScopeEquatorialSystem::usage = "GetScopeEquatorialSystem[theScope] Equatorial coordinate system used by this telescope.";
GetScopeFocalLength::usage = "GetScopeFocalLength[theScope] The telescope's focal length, meters";
GetScopeGuideRateDeclination::usage = "GetScopeGuideRateDeclination[theScope] The current Declination movement rate offset for telescope guiding (degrees/sec)";
GetScopeGuideRateRightAscension::usage = "GetScopeGuideRateRightAscension[theScope] The current Right Ascension movement rate offset for telescope guiding (degrees/sec)";
GetScopeInterfaceVersion::usage = "GetScopeInterfaceVersion[theScope] The interface version number that this device supports.";
GetScopeIsPulseGuiding::usage = "GetScopeIsPulseGuiding[theScope] True if a PulseGuide(GuideDirections, Int32) command is in progress, False otherwise";
GetScopeName::usage = "GetScopeName[theScope] The short name of the driver, for display purposes";
GetScopeRightAscension::usage = "GetScopeRightAscension[theScope] The right ascension (hours) of the telescope's current equatorial coordinates, in the coordinate system given by the EquatorialSystem property";
GetScopeRightAscensionRate::usage = "GetScopeRightAscensionRate[theScope] The right ascension tracking rate offset from sidereal (seconds per sidereal second, default = 0.0)";
GetScopeSideOfPier::usage = "GetScopeSideOfPier[theScope] Indicates the pointing state of the mount.";
GetScopeSiderealTime::usage = "GetScopeSiderealTime[theScope] The local apparent sidereal time from the telescope's internal clock (hours, sidereal)";
GetScopeSiteElevation::usage = "GetScopeSiteElevation[theScope] The elevation above mean sea level (meters) of the site at which the telescope is located";
GetScopeSiteLatitude::usage = "GetScopeSiteLatitude[theScope] The geodetic(map) latitude (degrees, positive North, WGS84) of the site at which the telescope is located.";
GetScopeSiteLongitude::usage = "GetScopeSiteLongitude[theScope] The longitude (degrees, positive East, WGS84) of the site at which the telescope is located.";
GetScopeSlewing::usage = "GetScopeSlewing[theScope] True if telescope is currently moving in response to one of the Slew methods or the MoveAxis(TelescopeAxes, Double) method, False at all other times.";
GetScopeSlewSettleTime::usage = "GetScopeSlewSettleTime[theScope] Specifies a post-slew settling time (sec.).";
GetScopeSupportedActions::usage = "GetScopeSupportedActions[theScope] Returns the list of custom action names supported by this driver.";
GetScopeTargetDeclination::usage = "GetScopeTargetDeclination[theScope] The declination (degrees, positive North) for the target of an equatorial slew or sync operation";
GetScopeTargetRightAscension::usage = "GetScopeTargetRightAscension[theScope] The right ascension (hours) for the target of an equatorial slew or sync operation";
GetScopeTracking::usage = "GetScopeTracking[theScope] The state of the telescope's sidereal tracking drive.";
GetScopeTrackingRate::usage = "GetScopeTrackingRate[theScope] The current tracking rate of the telescope's sidereal drive";
GetScopeTrackingRates::usage = "GetScopeTrackingRates[theScope] Returns a collection of supported DriveRates values that describe the permissible values of the TrackingRate property for this telescope type.";
GetScopeUTCDate::usage = "GetScopeUTCDate[theScope] The UTC date/time of the telescope's internal clock";

(* Telescope Methods *)
ScopeAbortSlew::usage="ScopeAbortSlew[scope] Stops a slew in progress.";
ScopeAction::usage="ScopeAction[scope, actionname, actionparameters] Invokes the specified device-specific custom action.";
ScopeAxisRates::usage="ScopeAxisRates[scope] A collection of rates at which the telescope may be moved about the specified axis by the MoveAxis(TelescopeAxes, Double) method.";
ScopeCanMoveAxis::usage="ScopeCanMoveAxis[scope] Shows whether the telescope can be controlled about the specified axis via the MoveAxis() method.";
ScopeChoose::usage="ScopeChoose[] The Choose() method returns the DriverID of the selected driver. Choose() allows you to optionally pass the DriverID of a current driver (you probably save this in the registry), and the corresponding telescope type is pre-selected in the Chooser's list. In this case, the OK button starts out enabled (lit-up); the assumption is that the pre-selected driver has already been configured.";
ScopeCommandBlind::usage="ScopeCommandBlind[scope, commandString, isRaw] Transmits an arbitrary string to the device and does not wait for a response. Optionally, protocol framing characters may be added to the string before transmission.";
ScopeCommandBool::usage="ScopeCommandBool[scope, commandString, isRaw] Transmits an arbitrary string to the device and waits for a boolean response. Optionally, protocol framing characters may be added to the string before transmission.";
ScopeCommandString::usage="ScopeCommandString[scope, commandString, isRaw] Transmits an arbitrary string to the device and waits for a string response. Optionally, protocol framing characters may be added to the string before transmission.";
ScopeDestinationSideOfPier::usage="ScopeDestinationSideOfPier[scope, ra, dec] Predict side of pier for German equatorial mounts";
ScopeDispose::usage="ScopeDispose[scope] Releases the unmanaged late bound COM object";
ScopeFindHome::usage="ScopeFindHome[scope] Locates the telescope home position (synchronous)";
ScopeMoveAxis::usage="ScopeMoveAxis[scope, axis, rate] Move the telescope in one axis at the given rate.";
ScopePark::usage="ScopePark[scope] Move the telescope to its park position, stop all motion (or restrict to a small safe range), and set AtPark to True.";
ScopePulseGuide::usage="ScopePulseGuide[scope, direction, durationMS] Moves the scope in the given direction for the given interval or time at the rate given by the corresponding guide rate property";
ScopeSetPark::usage="ScopeSetPark[scope] Sets the telescope Park position to the current telescope position.";
ScopeSetupDialog::usage="ScopeSetupDialog[scope] Launches a configuration dialogue box for the driver. The call will not return until the user clicks OK or cancel manually.";
ScopeSlewToAltAz::usage="ScopeSlewToAltAz[scope, az, alt] Move the telescope to the given local horizontal coordinates, return when slew is complete";
ScopeSlewToAltAzAsync::usage="ScopeSlewToAltAzAsync[scope, az, alt] This Method must be implemented if CanSlewAltAzAsync returns True.";
ScopeSlewToCoordinates::usage="ScopeSlewToCoordinates[scope, ra, dec] Move the telescope to the given equatorial coordinates, return when slew is complete";
ScopeSlewToCoordinatesAsync::usage="ScopeSlewToCoordinatesAsync[scope, ra, dec] Move the telescope to the given equatorial coordinates, return immediately after starting the slew.";
ScopeSlewToTarget::usage="ScopeSlewToTarget[scope] Move the telescope to the TargetRightAscension and TargetDeclination coordinates, return when slew complete.";
ScopeSlewToTargetAsync::usage="ScopeSlewToTargetAsync[scope] Move the telescope to the TargetRightAscension and TargetDeclination coordinates, returns immediately after starting the slew.";
ScopeSyncToAltAz::usage="ScopeSyncToAltAz[scope, az, alt] Matches the scope's local horizontal coordinates to the given local horizontal coordinates.";
ScopeSyncToCoordinates::usage="ScopeSyncToCoordinates[scope, ra, dec] Matches the scope's equatorial coordinates to the given equatorial coordinates.";
ScopeSyncToTarget::usage="ScopeSyncToTarget[scope] Matches the scope's equatorial coordinates to the given equatorial coordinates.";
ScopeUnpark::usage="ScopeUnpark[scope] Takes telescope out of the Parked state.";
ScopeSlewToObject::usage="ScopeSlewToObject[scope, objName] slews the scope to the named object like 'M42'\n";

(* Camera *)
(* deprecated *)
ActivateCameraCooler::usage="ActivateCameraCooler[camera, coolerOn] turns the camera's cooling system on or off";
GetCameraSensorTemperature::usage="GetCameraTemperature[camera] returns the camera temperature in degrees celcius";
SetCameraSensorTemperature::usage="SetCameraTemperature[camera, tempDegreesCelcius] sets the camera temperature in degrees celcius";
GetCameraAmbientTemperature::usage="GetCameraTemperature[camera] returns the camera temperature in degrees celcius";
GetCameraCoolerPowerLevel::usage="GetCameraCoolerPowerLevel[camera] returns the camera cooler power level in percent";
GetCameraBinning::usage="GetCameraBinning[camera] returns the camera's binning {binX, binY}";
SetCameraBinning::usage="SetCameraBinning[camera, xyBinning] sets the camera's binning: {binX, binY}";
GetCameraDimensions::usage="GetCameraDimensions[camera] returns the camera's sensor size in unbinned pixels {X, Y}";
SetSubframeDimensions::usage="SetSubframeDimensions[camera, xyOffsetAndSize] sets the camera's subframe dimensions: {{xOffset, yOffset}, {xSize, ySize}}";
GetSubframeDimensions::usage="GetSubframeDimensions[camera] returns the camera's subframe dimensions: {{xOffset, yOffset}, {xSize, ySize}}";
GetPixelDimensionsMicrons::usage="GetPixelDimensionsMicrons[camera] returns the camera's pixel dimensions in microns: {xSizeMicrons, ySizeMicrons}";
GetSensorType::usage="GetSensorType[camera] returns the camera's sensor type: Monochrome, Color, RGGB, CMYG, CMYG2, LRGB";
GetBayerOffset::usage="GetBayerOffset[camera] returns the {x, y} offset into the appropriate bayer matrix.  See GetSensorType[]";

(* Camera properties *)
GetCameraBayerOffsetX::usage = "GetCameraBayerOffsetX[theCamera] returns the bayer X offset index, Interface Version 2 only.";
GetCameraBayerOffsetY::usage = "GetCameraBayerOffsetY[theCamera] returns the Y offset of the Bayer matrix, as defined in SensorType, Interface Version 2 only.";
GetCameraBinX::usage = "GetCameraBinX[theCamera] returns the binning factor for the X axis.";
GetCameraBinY::usage = "GetCameraBinY[theCamera] returns the binning factor for the Y axis.";
GetCameraState::usage = "GetCameraState[theCamera] returns the current camera operational state";
GetCameraXSize::usage = "GetCameraXSize[theCamera] returns the width of the CCD camera chip in unbinned pixels.";
GetCameraYSize::usage = "GetCameraYSize[theCamera] returns the height of the CCD camera chip in unbinned pixels.";
GetCameraCanAbortExposure::usage = "GetCameraCanAbortExposure[theCamera] returns True if the camera can abort exposures; False if not.";
GetCameraCanAsymmetricBin::usage = "GetCameraCanAsymmetricBin[theCamera] returns a flag showing whether this camera supports asymmetric binning";
GetCameraCanFastReadout::usage = "GetCameraCanFastReadout[theCamera] returns True if camera has a fast readout mode, Interface Version 2 only";
GetCameraCanGetCoolerPower::usage = "GetCameraCanGetCoolerPower[theCamera] if True, the camera's cooler power setting can be read.";
GetCameraCanPulseGuide::usage = "GetCameraCanPulseGuide[theCamera] returns a flag indicating whether this camera supports pulse guiding.";
GetCameraCanSetCCDTemperature::usage = "GetCameraCanSetCCDTemperature[theCamera] returns a flag indicating whether this camera supports setting the CCD temperature.";
GetCameraCanStopExposure::usage = "GetCameraCanStopExposure[theCamera] returns a flag indicating whether this camera can stop an exposure that is in progress.";
GetCameraCCDTemperature::usage = "GetCameraCCDTemperature[theCamera] returns the current CCD temperature in degrees Celsius.";
GetCameraConnected::usage = "GetCameraConnected[theCamera] returns True if connected to the device hardware. Returns False if disconnected from the device hardware. This reports the current hardware state.";
GetCameraCoolerOn::usage = "GetCameraCoolerOn[theCamera] turns on and off the camera cooler, and returns the current on/off state.";
GetCameraCoolerPower::usage = "GetCameraCoolerPower[theCamera] returns the present cooler power level, in percent.";
GetCameraDescription::usage = "GetCameraDescription[theCamera] returns a description of the device, such as manufacturer and model number. Any ASCII characters may be used.";
GetCameraDriverInfo::usage = "GetCameraDriverInfo[theCamera] descriptive and version information about this ASCOM driver.";
GetCameraDriverVersion::usage = "GetCameraDriverVersion[theCamera] a string containing only the major and minor version of the driver.";
GetCameraElectronsPerADU::usage = "GetCameraElectronsPerADU[theCamera] returns the gain of the camera in photoelectrons per A/D unit.";
GetCameraExposureMax::usage = "GetCameraExposureMax[theCamera] returns the maximum exposure time supported by StartExposure, Interface Version 2 only";
GetCameraExposureMin::usage = "GetCameraExposureMin[theCamera] minimum exposure time, Interface Version 2 only";
GetCameraExposureResolution::usage = "GetCameraExposureResolution[theCamera] exposure resolution, Interface Version 2 only";
GetCameraFastReadout::usage = "GetCameraFastReadout[theCamera] fast readout mode, Interface Version 2 only";
GetCameraFullWellCapacity::usage = "GetCameraFullWellCapacity[theCamera] reports the full well capacity of the camera in electrons, at the current camera settings (binning, SetupDialog settings, etc.)";
GetCameraGain::usage = "GetCameraGain[theCamera] the camera's gain (GAIN VALUE MODE) OR the index of the selected camera gain description in the Gains array (GAINS INDEX MODE)";
GetCameraGainMax::usage = "GetCameraGainMax[theCamera] maximum Gain value of that this camera supports";
GetCameraGainMin::usage = "GetCameraGainMin[theCamera] minimum Gain value of that this camera supports";
GetCameraGains::usage = "GetCameraGains[theCamera] list of Gain names supported by the camera";
GetCameraHasShutter::usage = "GetCameraHasShutter[theCamera] returns a flag indicating whether this camera has a mechanical shutter";
GetCameraHeatSinkTemperature::usage = "GetCameraHeatSinkTemperature[theCamera] returns the current heat sink temperature (called \"ambient temperature\" by some manufacturers) in degrees Celsius.";
GetCameraImageArray::usage = "GetCameraImageArray[theCamera] returns a safearray of int of size NumX * NumY containing the pixel values from the last exposure.";
GetCameraImageArrayVariant::usage = "GetCameraImageArrayVariant[theCamera] returns a safearray of Variant of size NumX * NumY containing the pixel values from the last exposure.";
GetCameraImageReady::usage = "GetCameraImageReady[theCamera] returns a flag indicating whether the image is ready to be downloaded from the camera";
GetCameraInterfaceVersion::usage = "GetCameraInterfaceVersion[theCamera] the interface version number that this device supports.";
GetCameraIsPulseGuiding::usage = "GetCameraIsPulseGuiding[theCamera] returns a flag indicating whether the camera is currently in a PulseGuide operation.";
GetCameraLastExposureDuration::usage = "GetCameraLastExposureDuration[theCamera] reports the actual exposure duration in seconds (i.e. shutter open time).";
GetCameraLastExposureStartTime::usage = "GetCameraLastExposureStartTime[theCamera] reports the actual exposure start in the FITS-standard CCYY-MM-DDThh:mm:ss[.sss...] format. The time must be UTC.";
GetCameraMaxADU::usage = "GetCameraMaxADU[theCamera] reports the maximum ADU value the camera can produce.";
GetCameraMaxBinX::usage = "GetCameraMaxBinX[theCamera] returns the maximum allowed binning for the X camera axis";
GetCameraMaxBinY::usage = "GetCameraMaxBinY[theCamera] returns the maximum allowed binning for the Y camera axis";
GetCameraName::usage = "GetCameraName[theCamera] the short name of the driver, for display purposes";
GetCameraNumX::usage = "GetCameraNumX[theCamera] sets the subframe width. Also returns the current value.";
GetCameraNumY::usage = "GetCameraNumY[theCamera] sets the subframe height. Also returns the current value.";
GetCameraOffset::usage = "GetCameraOffset[theCamera] the camera's offset (OFFSET VALUE MODE) OR the index of the selected camera offset description in the Offsets array (OFFSETS INDEX MODE)";
GetCameraOffsetMax::usage = "GetCameraOffsetMax[theCamera] maximum Offset value of that this camera supports";
GetCameraOffsetMin::usage = "GetCameraOffsetMin[theCamera] minimum Offset value of that this camera supports";
GetCameraOffsets::usage = "GetCameraOffsets[theCamera] list of Offset names supported by the camera";
GetCameraPercentCompleted::usage = "GetCameraPercentCompleted[theCamera] percent completed, Interface Version 2 only";
GetCameraPixelSizeX::usage = "GetCameraPixelSizeX[theCamera] returns the width of the CCD chip pixels in microns.";
GetCameraPixelSizeY::usage = "GetCameraPixelSizeY[theCamera] returns the height of the CCD chip pixels in microns.";
GetCameraReadoutMode::usage = "GetCameraReadoutMode[theCamera] readout mode, Interface Version 2 only";
GetCameraReadoutModes::usage = "GetCameraReadoutModes[theCamera] list of available readout modes, Interface Version 2 only";
GetCameraSensorName::usage = "GetCameraSensorName[theCamera] sensor name, Interface Version 2 only";
GetCameraSensorType::usage = "GetCameraSensorType[theCamera] type of colour information returned by the camera sensor, Interface Version 2 only";
GetCameraCCDTemperature::usage = "GetCameraCCDTemperature[theCamera] sets the camera cooler setpoint in degrees Celsius, and returns the current setpoint.";
GetCameraStartX::usage = "GetCameraStartX[theCamera] sets the subframe start position for the X axis (0 based) and returns the current value.";
GetCameraStartY::usage = "GetCameraStartY[theCamera] sets the subframe start position for the Y axis (0 based). Also returns the current value.";
GetCameraSubExposureDuration::usage = "GetCameraSubExposureDuration[theCamera] camera's sub-exposure interval";
GetCameraSupportedActions::usage = "GetCameraSupportedActions[theCamera] returns the list of custom action names supported by this driver.";
SetCameraBinX::usage = "SetCameraBinX[theCamera, theValue] sets the binning factor for the X axis, also returns the current value.";
SetCameraBinY::usage = "SetCameraBinY[theCamera, theValue] sets the binning factor for the Y axis, also returns the current value.";
SetCameraConnected::usage = "SetCameraConnected[theCamera, theValue] set True to connect to the device hardware. Set False to disconnect from the device hardware. You can also read the property to check whether it is connected. This reports the current hardware state.";
SetCameraCoolerOn::usage = "SetCameraCoolerOn[theCamera, theValue] turns on and off the camera cooler, and returns the current on/off state.";
SetCameraFastReadout::usage = "SetCameraFastReadout[theCamera, theValue] fast readout mode, Interface Version 2 only";
SetCameraGain::usage = "SetCameraGain[theCamera, theValue] the camera's gain (GAIN VALUE MODE) OR the index of the selected camera gain description in the Gains array (GAINS INDEX MODE)";
SetCameraNumX::usage = "SetCameraNumX[theCamera, theValue] sets the subframe width. Also returns the current value.";
SetCameraNumY::usage = "SetCameraNumY[theCamera, theValue] sets the subframe height. Also returns the current value.";
SetCameraOffset::usage = "SetCameraOffset[theCamera, theValue] the camera's offset (OFFSET VALUE MODE) OR the index of the selected camera offset description in the Offsets array (OFFSETS INDEX MODE)";
SetCameraReadoutMode::usage = "SetCameraReadoutMode[theCamera, theValue] readout mode, Interface Version 2 only";
SetCameraCCDTemperature::usage = "SetCameraCCDTemperature[theCamera, theValue] sets the camera cooler setpoint in degrees Celsius, and returns the current setpoint.";
SetCameraStartX::usage = "SetCameraStartX[theCamera, theValue] sets the subframe start position for the X axis (0 based) and returns the current value.";
SetCameraStartY::usage = "SetCameraStartY[theCamera, theValue] sets the subframe start position for the Y axis (0 based). Also returns the current value.";
SetCameraSubExposureDuration::usage = "SetCameraSubExposureDuration[theCamera, theValue] camera's sub-exposure interval";
SetAskDarkFrame::usage = "SetAskDarkFrame[ask] sets an internal flag to manually cover and uncover the scope objective.\n";

(* Utilities for capturing calibration frames *)
AcquireDarkFrameSequence::usage = "AcquireDarkFrameSequence[theCamera, durationSec, count, prefix, outputDir] gets a sequence of dark frames";
AcquireBiasFrameSequence::usage = "AcquireBiasFrameSequence[theCamera, count, prefix, outputDir] gets a sequence of bias frames";
AcquireFlatFrameSequence::usage = "AcquireFlatFrameSequence[theCamera, count, exposureTime, prefix, outputDir] gets a sequence of optimal flat frames";
PlotFlatFrameResponse::usage = "PlotFlatFrameResponse[theCamera, maxTimeSeconds, count] plots count median image values in the range 0 to maxTimeSeconds.\nThis is used to determine optimal flat frame exposure times.";
DetermineOptimalFlatExposure::usage = "DetermineOptimalFlatExposure[theCamera, maxTimeSeconds] in the linear range 0 to maxTimeSeconds, this will solve for the optimal flat frame exposure time.";

(* Filter Wheel *)
SetFilterWheelPosition::usage="SetFilterWheelPosition[filterwheel, n] selects the filter n (1 based)\nSetFilterWheelPosition[filterwheel, name] selects the filter name";
GetFilterWheelPosition::usage="GetFilterWheelPosition[filterwheel] returns the current filter position (1 based)";
GetFilterWheelCurrentName::usage="GetFilterWheelCurrentName[filterwheel] returns the name of the current filter";
GetFilterWheelNames::usage="GetFilterWheelNames[filterwheel] returns the names of the current filters";
GetFilterWheelFocusOffsets::usage="GetFilterWheelFocusOffsets[filterwheel] returns the focus offsets for the filter wheel";
GetFilterWheelFocusOffset::usage="GetFilterWheelFocusOffset[filterwheel, theFilterPos] returns the absolute focus offset for a given filter.\nGetFilterWheelFocusOffset[filterwheel, name] returns the absolute focus offset for a given filter name.";

(* Focuser *)
HaltFocuser::usage="HaltFocuser[focuser] immediately stops any focuser movement";
MoveFocuser::usage="MoveFocuser[focuser, amountOrPosition] moves the focuser by amountOrPosition depending on the focuser type.";
IsFocuserAbsolute::usage="IsFocuserAbsolute[focuser] returns True if the focuser is capable of absolute position; that is, being commanded to a specific step location.";
GetFocuserMaxIncrement::usage="GetFocuserMaxIncrement[focuser] returns maximum increment size allowed by the focuser; i.e. the maximum number of steps allowed in one move operation.";
GetFocuserMaxStep::usage="GetFocuserMaxStep[focuser] returns maximum step position permitted.";
GetFocuserPosition::usage="GetFocuserPosition[focuser] returns current focuser position, in steps.";
GetFocuserStepSize::usage="GetFocuserStepSize[focuser] returns step size (microns) for the focuser.";

(* Dome *)
DomeAltitude::usage="The altitude (degrees, horizon zero and increasing positive to 90 zenith) of the part of the sky that the observer wishes to observe.";
DomeAtHome::usage="True when the dome is in the home position. Raises an error if not supported.  This is normally used following a FindHome() operation. The value is reset with any azimuth slew operation that moves the dome away from the home position.  AtHome may optionally also become True during normal slew operations, if the dome passes through the home position and the dome controller hardware is capable of detecting that; or at the end of a slew operation if the dome comes to rest at the home position.";
DomeAtPark::usage="True if the dome is in the programmed park position.";
DomeAzimuth::usage="The dome azimuth (degrees, North zero and increasing clockwise, i.e., 90 East, 180 South, 270 West). North is True north and not magnetic north.";
DomeCanFindHome::usage="True if driver can perform a search for home position.";
DomeCanPark::usage="True if the driver is capable of parking the dome.";
DomeCanSetAltitude::usage="True if driver is capable of setting dome altitude.";
DomeCanSetAzimuth::usage="True if driver is capable of rotating the dome (or controlling the roof mechanism) in order to observe at a given azimuth.";
DomeCanSetPark::usage="True if the driver can set the dome park position.";
DomeCanSetShutter::usage="True if the driver is capable of opening and closing the shutter or roof mechanism.";
DomeCanSlave::usage="True if the dome hardware supports slaving to a telescope.";
DomeCanSyncAzimuth::usage="True if the driver is capable of synchronizing the dome azimuth position using the SyncToAzimuth(Double) method.";
DomeConnected::usage="Set True to connect to the device hardware. Set False to disconnect from the device hardware. You can also read the property to check whether it is connected. This reports the current hardware state.";
DomeDescription::usage="Returns a description of the device, such as manufacturer and model number. Any ASCII characters may be used.";
DomeDriverInfo::usage="Descriptive and version information about this ASCOM driver.";
DomeDriverVersion::usage="A string containing only the major and minor version of the driver.";
DomeInterfaceVersion::usage="The interface version number that this device supports.";
DomeName::usage="The short name of the driver, for display purposes";
DomeShutterStatus::usage="Gets the status of the dome shutter or roof structure.";
DomeSlaved::usage="True if the dome is slaved to the telescope in its hardware, else False.";
DomeSlewing::usage="True if any part of the dome is currently moving, False if all dome components are stationary.";
DomeSupportedActions::usage="Returns the list of custom action names supported by this driver.";
DomeAbortSlew::usage="Immediately stops any and all movement.";
DomeAction::usage="Invokes the specified device-specific custom action.";
DomeChoose::usage="Shows the ASCOM Chooser to select a Dome.";
DomeCloseShutter::usage="Close the shutter or otherwise shield the telescope from the sky.";
DomeCommandBlind::usage="Transmits an arbitrary string to the device and does not wait for a response. Optionally, protocol framing characters may be added to the string before transmission.";
DomeCommandBool::usage="Transmits an arbitrary string to the device and waits for a boolean response. Optionally, protocol framing characters may be added to the string before transmission.";
DomeCommandString::usage="Transmits an arbitrary string to the device and waits for a string response. Optionally, protocol framing characters may be added to the string before transmission.";
DomeDispose::usage="Releases the unmanaged late bound COM object";
DomeFindHome::usage="Start operation to search for the dome home position.";
DomeOpenShutter::usage="Open shutter or otherwise expose telescope to the sky.";
DomePark::usage="Rotate dome in azimuth to park position.";
DomeSetPark::usage="Set the current azimuth position of dome to the park position.";
DomeSetupDialog::usage="Launches a configuration dialogue box for the driver. The call will not return until the user clicks OK or cancel manually.";
DomeSlewToAltitude::usage="Ensure that the requested viewing altitude is available for observing.";
DomeSlewToAzimuth::usage="Ensure that the requested viewing azimuth is available for observing. The method should not block and the slew operation should complete asynchronously.";
DomeSyncToAzimuth::usage="Synchronize the current position of the dome to the given azimuth.";


(* Higher Level Functions *)
SelectDevice::usage="SelectDevice[deviceName] allows you to select an imaging device.  deviceName = {\"Camera\", \"Focuser\", \"FilterWheel\"}";
SetupDevice::usage="SetupDevice[device] displays the device's setup dialog";
ConnectDevice::usage="ConnectDevice[device] connects the device.";
DisconnectDevice::usage="DisconnectDevice[device] disconnects from the device";
AcquireImage::usage="AcquireImage[durationseconds, shutterOpen] acquires a light or dark image of durationseconds length";
AcquireImageToFile::usage="AcquireImageToFile[durationseconds, shutterOpen, fileName] acquires a light or dark image of durationseconds length directly to the fileName";
AcquireImageToFITSFile::usage="AcquireImageToFITSFile[durationseconds, shutterOpen, fileName] acquires a light or dark image of durationseconds length directly to a FITS fileName, with FITS meta information added";
AcquireFITSImageSequence::usage="AcquireFITSImageSequence[theCamera, durationseconds, shutterOpen, count, prefix, outputDir] acquires count light or dark FITS images of durationseconds length directly to outputDir";

AcquireRGBSequence::usage="AcquireRGBSequence[theCamera, theFilterWheel, frameExposure, frameCount, binning, outputDir] acquires sequences of images through RGB filters";
AcquireLRGBSequence::usage="AcquireLRGBSequence[theCamera, theFilterWheel, frameExposure, frameCount, outputDir] acquires sequences of low res RGB and high res Luminance images";
AcquireDarkBiasFrames::usage="AcquireDarkBiasFrames[camera, filterwheel, exposure, count, outputDir] acquires count dark and bias calibration frames";

(* deprecated
AcquireImageSequence::usage="AcquireImageSequence[theCamera, durationseconds, shutterOpen, count, prefix, outputDir] acquires count light or dark FITS images of durationseconds length directly to outputDir";
AcquireFilteredImageSequence::usage="AcquireFilteredImageSequence[theCamera, theFilterWheel, theFocuser, durationseconds, shutterOpen, count, prefix, outputDir, filterName] acquires count light or dark FITS images of durationseconds length directly to outputDir, with a given filter";
AcquireFilteredCalibrationSet::usage="AcquireFilteredCalibrationSet[theCamera, theFilterWheel, theFocuser, exposure, flatexposure, biasexposure, frameCount, objectName, outputDir, filterSet] acquires calibration frames for a given set of filters";
AcquireFilteredImageSet::usage="AcquireFilteredImageSet[theCamera, theFilterWheel, theFocuser, exposure, frameCount, objectName, outputDir, filterSet] acquires imaging frames for a given filter set.";
*)

Begin["`Private`"]

InitDeviceInterface[]:=
Module[{},
  InstallNET[];
  LoadNETAssembly /@  Map["ASCOM." <> # &, {"Astrometry", "Attributes", "DriverAccess", "Exceptions", "Helper", "Helper2", "Interfaces", "Utilities"}];
  LoadNETType["ASCOM.DriverAccess.Camera"];
  LoadNETType["ASCOM.DriverAccess.FilterWheel"];
  LoadNETType["ASCOM.DriverAccess.Focuser"];
  LoadNETType["ASCOM.DriverAccess.Dome"];
];

InitDeviceInterface32[]:=
Module[{},
  InstallNET["Force32Bit" -> True];
  LoadNETAssembly /@  Map["ASCOM." <> # &, {"Astrometry", "Attributes", "DriverAccess", "Exceptions", "Helper", "Helper2", "Interfaces", "Utilities"}];
  LoadNETType["ASCOM.DriverAccess.Camera"];
  LoadNETType["ASCOM.DriverAccess.FilterWheel"];
  LoadNETType["ASCOM.DriverAccess.Focuser"];
  LoadNETType["ASCOM.DriverAccess.Dome"];
];

(*
  Choose a device (camera, focuser, filterwheel, dome)
*)
SelectDevice[deviceName_String]:=
Module[{theDevice},
  theDevice = NETNew["ASCOM.DriverAccess." <> deviceName, ToExpression[deviceName<>"`Choose[\"\"]"]];
  theDevice@Connected = True;
  theDevice
]

(*
  Bring up the device (camera, focuser, filterwheel, dome) setup dialog
*)
SetupDevice[theDevice_]:=
Module[{},
  theDevice@Connected = False;
  theDevice@SetupDialog[];
  theDevice@Connected = True;
];

(*
  Connect and disconnect from the device
*)
ConnectDevice[theDevice_]:=
Module[{},
  theDevice@Connected = True;
]

DisconnectDevice[theDevice_]:=
Module[{},
  theDevice@Connected = False;
];

(* Scope Functions *)
(*
*)
(* Properties *)
(* The alignment mode of the mount. *)
ScopeAlignmentMode[theScope_]:=theScope@AlignmentMode;

(* The Altitude above the local horizon of the telescopes current position (degrees, positive up) *)
ScopeAltitude[theScope_]:=theScope@Altitude;

(* The area of the telescopes aperture, taking into account any obstructions (square meters) *)
ScopeApertureArea[theScope_]:=theScope@ApertureArea;

(* The telescope effective aperture diameter (meters) *)
ScopeApertureDiameter[theScope_]:=theScope@ApertureDiameter;

(* True if the telescope is stopped in the Home position. Set only following a FindHome() operation, and reset with any slew operation. This property must be False if the telescope does not support homing. *)
ScopeAtHome[theScope_]:=theScope@AtHome;

(* True if the telescope has been put into the parked state by the see Park() method. Set False by calling the Unpark() method. *)
ScopeAtPark[theScope_]:=theScope@AtPark;

(* The azimuth at the local horizon of the telescope current position (degrees, North-referenced, positive East/clockwise). *)
ScopeAzimuth[theScope_]:=theScope@Azimuth;

(* True if this telescope is capable of programmed finding its home position (FindHome() method). *)
ScopeCanFindHome[theScope_]:=theScope@CanFindHome;

(* True if this telescope is capable of programmed parking (Park()method) *)
ScopeCanPark[theScope_]:=theScope@CanPark;

(* True if this telescope is capable of software-pulsed guiding (via the PulseGuide(GuideDirections, Int32) method) *)
ScopeCanPulseGuide[theScope_]:=theScope@CanPulseGuide;

(* True if the DeclinationRate property can be changed to provide offset tracking in the declination axis. *)
ScopeCanSetDeclinationRate[theScope_]:=theScope@CanSetDeclinationRate;

(* True if the guide rate properties used for PulseGuide(GuideDirections, Int32) can be adjusted. *)
ScopeCanSetGuideRates[theScope_]:=theScope@CanSetGuideRates;

(* True if this telescope is capable of programmed setting of its park position (SetPark() method) *)
ScopeCanSetPark[theScope_]:=theScope@CanSetPark;

(* True if the SideOfPier property can be set, meaning that the mount can be forced to flip. *)
ScopeCanSetPierSide[theScope_]:=theScope@CanSetPierSide;

(* True if the RightAscensionRate property can be changed to provide offset tracking in the right ascension axis. *)
ScopeCanSetRightAscensionRate[theScope_]:=theScope@CanSetRightAscensionRate;

(* True if the Tracking property can be changed, turning telescope sidereal tracking on and off. *)
ScopeCanSetTracking[theScope_]:=theScope@CanSetTracking;

(* True if this telescope is capable of programmed slewing (synchronous or asynchronous) to equatorial coordinates *)
ScopeCanSlew[theScope_]:=theScope@CanSlew;

(* True if this telescope is capable of programmed slewing (synchronous or asynchronous) to local horizontal coordinates *)
ScopeCanSlewAltAz[theScope_]:=theScope@CanSlewAltAz;

(* True if this telescope is capable of programmed asynchronous slewing to local horizontal coordinates *)
ScopeCanSlewAltAzAsync[theScope_]:=theScope@CanSlewAltAzAsync;

(* True if this telescope is capable of programmed asynchronous slewing to equatorial coordinates. *)
ScopeCanSlewAsync[theScope_]:=theScope@CanSlewAsync;

(* True if this telescope is capable of programmed syncing to equatorial coordinates. *)
ScopeCanSync[theScope_]:=theScope@CanSync;

(* True if this telescope is capable of programmed syncing to local horizontal coordinates *)
ScopeCanSyncAltAz[theScope_]:=theScope@CanSyncAltAz;

(* True if this telescope is capable of programmed unparking (Unpark() method). *)
ScopeCanUnpark[theScope_]:=theScope@CanUnpark;

(* Set True to connect to the device hardware. Set False to disconnect from the device hardware. You can also read the property to check whether it is connected. This reports the current hardware state. *)
ScopeConnected[theScope_]:=theScope@Connected;
ScopeConnected[theScope_, state_Boolean]:=Block[{}, theScope@Connected = state; theScope@Connected];

(* The declination (degrees) of the telescopes current equatorial coordinates, in the coordinate system given by the EquatorialSystem property. Reading the property will raise an error if the value is unavailable. *)
ScopeDeclination[theScope_]:=theScope@Declination;

(* The declination tracking rate (arcseconds per SI second, default = 0.0) *)
ScopeDeclinationRate[theScope_]:=theScope@DeclinationRate;
ScopeDeclinationRate[theScope_, rate_Real]:=Block[{}, theScope@DeclinationRate = rate; theScope@DeclinationRate];

(* Returns a description of the device, such as manufacturer and model number. Any ASCII characters may be used. *)
ScopeDescription[theScope_]:=theScope@Description;

(* True if the telescope or driver applies atmospheric refraction to coordinates. *)
ScopeDoesRefraction[theScope_]:=theScope@DoesRefraction;
ScopeDoesRefraction[theScope_, doesRefraction_Boolean]:=Block[{}, theScope@DoesRefraction = doesRefraction; theScope@DoesRefraction];

(* Descriptive and version information about this ASCOM driver. *)
ScopeDriverInfo[theScope_]:=theScope@DriverInfo;

(* A string containing only the major and minor version of the driver. *)
ScopeDriverVersion[theScope_]:=theScope@DriverVersion;

(* Equatorial coordinate system used by this telescope. *)
ScopeEquatorialSystem[theScope_]:=theScope@EquatorialSystem;

(* The telescopes focal length, meters *)
ScopeFocalLength[theScope_]:=theScope@FocalLength;

(* ScopeGuideRateDeclination *)
ScopeGuideRateDeclination[theScope_]:=theScope@GuideRateDeclination;
ScopeGuideRateDeclination[theScope_, rate_Real]:=Block[{}, theScope@GuideRateDeclination = rate; theScope@GuideRateDeclination];

(* The current Right Ascension movement rate offset for telescope guiding (degrees/sec) *)
ScopeGuideRateRightAscension[theScope_]:=theScope@GuideRateRightAscension;
ScopeGuideRateRightAscension[theScope_, rate_Real]:=Block[{}, theScope@GuideRateRightAscension = rate; theScope@GuideRateRightAscension];

(* The interface version number that this device supports. *)
ScopeInterfaceVersion[theScope_]:=theScope@InterfaceVersion;

(* True if a PulseGuide(GuideDirections, Int32) command is in progress, False otherwise *)
ScopeIsPulseGuiding[theScope_]:=theScope@PulseGuiding;

(* The short name of the driver, for display purposes *)
ScopeName[theScope_]:=theScope@Name;

(* The right ascension (hours) of the telescope current equatorial coordinates, in the coordinate system given by the EquatorialSystem property *)
ScopeRightAscension[theScope_]:=theScope@RightAscension;

(* ScopeRightAscensionRate *)
ScopeRightAscensionRate[theScope_]:=theScope@RightAscensionRate;
ScopeRightAscensionRate[theScope_, theRate_Real]:=Block[{},theScope@RightAscensionRate = theRate;theScope@RightAscensionRate];

(* Indicates the pointing state of the mount. *)
(*
  pierEast	  0	  Normal pointing state - Mount on the East side of pier (looking West)
  pierUnknown	-1	Unknown or indeterminate.
  pierWest	  1	  Through the pole pointing state - Mount on the West side of pier (looking East)
*)
ScopeSideOfPier[theScope_]:=theScope@SideOfPier;
ScopeSideOfPier[theScope_, pierSide_]:=Block[{},theScope@SideOfPier=pierSide;theScope@SideOfPier];

(* The local apparent sidereal time from the telescope internal clock (hours, sidereal) *)
ScopeSiderealTime[theScope_]:=theScope@SiderealTime;

(* The elevation above mean sea level (meters) of the site at which the telescope is located *)
ScopeSiteElevation[theScope_]:=theScope@SiteElevation;
ScopeSiteElevation[theScope_, theElevation_Real]:=Block[{}, theScope@SiteElevation = theElevation;theScope@SiteElevation];

(* The geodetic(map) latitude (degrees, positive North, WGS84) of the site at which the telescope is located. *)
ScopeSiteLatitude[theScope_]:=theScope@SiteLatitude;
ScopeSiteLatitude[theScope_, theLatitude_Real]:=Block[{},theScope@SiteLatitude=theLatitude;theScope@SiteLatitude];

(* The longitude (degrees, positive East, WGS84) of the site at which the telescope is located. *)
ScopeSiteLongitude[theScope_]:=theScope@SiteLongitude;
ScopeSiteLongitude[theScope_, theLongitude_Real]:=Block[{}, theScope@SiteLongitude = theLongitude; theScope@SiteLongitude];

(* True if telescope is currently moving in response to one of the Slew methods or the MoveAxis(TelescopeAxes, Double) method, False at all other times. *)
ScopeSlewing[theScope_]:=theScope@Slewing;

(* Specifies a post-slew settling time (sec.). *)
ScopeSlewSettleTime[theScope_]:=theScope@SlewSettleTime;
ScopeSlewSettleTime[theScope_, settleTime_Integer]:=Block[{}, theScope@SlewSettleTime = settleTime; theScope@SlewSettleTime];

(* Returns the list of custom action names supported by this driver. *)
ScopeSupportedActions[theScope_]:=theScope@SupportedActions;

(* The declination (degrees, positive North) for the target of an equatorial slew or sync operation *)
(* 
  Setting this property will raise an error if the given value is outside the range -90 to +90 degrees. 
  Reading the property will raise an error if the value has never been set or is otherwise unavailable.
*)
ScopeTargetDeclination[theScope_]:=theScope@TargetDeclination;
ScopeTargetDeclination[theScope_, targetDec_Real]:=Block[{}, theScope@TargetDeclination = targetDec; theScope@TargetDeclination];

(* The right ascension (hours) for the target of an equatorial slew or sync operation *)
ScopeTargetRightAscension[theScope_]:=theScope@TargetRightAscension;
ScopeTargetRightAscension[theScope_, targetRA_Real]:=Block[{}, theScope@TargetRightAscension = targetRA; theScope@TargetRightAscension];

(* The state of the telescope sidereal tracking drive. *)
ScopeTracking[theScope_]:=theScope@Tracking;
ScopeTracking[theScope_, isTracking_Boolean]:=Block[{}, theScope@Tracking = isTracking; theScope@Tracking];

(* The current tracking rate of the telescope sidereal drive *)
(*
  driveSidereal	0	Sidereal tracking rate (15.041 arcseconds per second).
  driveLunar	  1	Lunar tracking rate (14.685 arcseconds per second).
  driveSolar	  2	Solar tracking rate (15.0 arcseconds per second).
  driveKing	    3	King tracking rate (15.0369 arcseconds per second).
*)
ScopeTrackingRate[theScope_]:=theScope@TrackingRate;
ScopeTrackingRate[theScope_, theRate_Integer]:=Block[{}, theScope@TrackingRate = theRate; theScope@TrackingRate];

(* Returns a collection of supported DriveRates values that describe the permissible values of the TrackingRate property for this telescope type. *)
ScopeTrackingRates[theScope_]:=theScope@TrackingRates;

(* The UTC date/time of the telescope internal clock *)
ScopeUTCDate[theScope_]:=theScope@UTCDate;

(* Methods *)
(* Stops a slew in progress. *)
ScopeAbortSlew[theScope_]:=theScope@AbortSlew[];	

(* Invokes the specified device-specific custom action. *)
ScopeAction[theScope_, actionName_String, actionParameters_String]:=theScope@Action[actionName, actionParameters];

(* A collection of rates at which the telescope may be moved about the specified axis by the MoveAxis(TelescopeAxes, Double) method. *)
(* 
  axisPrimary	  0	Primary axis (e.g., Right Ascension or Azimuth).
  axisSecondary	1	Secondary axis (e.g., Declination or Altitude).
  axisTertiary	2	Tertiary axis (e.g. imager rotator/de-rotator).
*)
ScopeAxisRates[theScope_]:=theScope@AxisRates[];

(* Shows whether the telescope can be controlled about the specified axis via the MoveAxis() method. *)
(* 
  axisPrimary	  0	Primary axis (e.g., Right Ascension or Azimuth).
  axisSecondary	1	Secondary axis (e.g., Declination or Altitude).
  axisTertiary	2	Tertiary axis (e.g. imager rotator/de-rotator).
*)
ScopeCanMoveAxis[theScope_]:=theScope@CanMoveAxis[];

(* 
  The Choose() method returns the DriverID of the selected driver. Choose() allows you to optionally pass the DriverID of a current driver 
  (you probably save this in the registry), and the corresponding telescope type is pre-selected in the Chooser list. In this case, the OK 
  button starts out enabled (lit-up); the assumption is that the pre-selected driver has already been configured. 
*)
(* TODO STATIC METHOD *)
ScopeChoose[]:=Choose[];

(* Transmits an arbitrary string to the device and does not wait for a response. Optionally, protocol framing characters may be added to the string before transmission. *)
ScopeCommandBlind[theScope_, theCommand_String, isRaw_]:=theScope@CommandBlind[theCommand, isRaw];

(* Transmits an arbitrary string to the device and waits for a boolean response. Optionally, protocol framing characters may be added to the string before transmission. *)
ScopeCommandBool[theScope_, theCommand_String, isRaw_]:=theScope@CommandBool[theCommand, isRaw];

(* Transmits an arbitrary string to the device and waits for a string response. Optionally, protocol framing characters may be added to the string before transmission. *)
ScopeCommandString[theScope_, theCommand, isRaw]:=theScope@CommandString[theCommand, isRaw];

(* Predict side of pier for German equatorial mounts.  Returns PierSide *)
ScopeDestinationSideOfPier[theScope_, theRA_Real, theDEC_Real]:=theScope@DestinationSideOfPier[theRA, theDEC];

(* Releases the unmanaged late bound COM object *)
ScopeDispose[theScope_]:=theScope@Dispose[];

(* Locates the telescope "home" position (synchronous) *)
ScopeFindHome[theScope_]:=theScope@FindHome[];

(* Move the telescope in one axis at the given rate. *)
(* 
  axisPrimary	  0	Primary axis (e.g., Right Ascension or Azimuth).
  axisSecondary	1	Secondary axis (e.g., Declination or Altitude).
  axisTertiary	2	Tertiary axis (e.g. imager rotator/de-rotator).
*)
ScopeMoveAxis[theScope_, scopeAxis_, axisRate_Real]:=theScope@MoveAxis[scopeAxis, axisRate];

(* Move the telescope to its park position, stop all motion (or restrict to a small safe range), and set AtPark to True. *)
ScopePark[theScope_]:=theScope@Park[];

(* Moves the scope in the given direction for the given interval or time at the rate given by the corresponding guide rate property *)
(*
  guideNorth	0	North (+ declination/altitude).
  guideSouth	1	South (- declination/altitude).
  guideEast	  2	East (+ right ascension/azimuth).
  guideWest	  3	West (- right ascension/azimuth)
  guideDuration : The duration of the guide-rate motion (milliseconds)
*)
ScopePulseGuide[theScope_, guideDirection_, guideDuration_]:=theScope@PulseGuide[guideDirection, guideDuration];

(* Sets the telescope Park position to the current telescope position. *)
ScopeSetPark[theScope_]:=theScope@SetPark[];

(* Launches a configuration dialogue box for the driver. The call will not return until the user clicks OK or cancel manually. *)
ScopeSetupDialog[theScope_]:=theScope@SetupDialog[];

(* Move the telescope to the given local horizontal coordinates, return when slew is complete *)
ScopeSlewToAltAz[theScope_, az_Real, alt_Real]:=theScope@SlewToAltAz[az, alt];

(* This Method must be implemented if CanSlewAltAzAsync returns True. *)
ScopeSlewToAltAzAsync[theScope_, az_Real, alt_Real]:=theScope@SlewToAltAzAsync[az, alt];

(* Move the telescope to the given equatorial coordinates, return when slew is complete *)
ScopeSlewToCoordinates[theScope_, ra_Real, dec_Real]:=theScope@SlewToCoordinates[ra, dec];

(* Move the telescope to the given equatorial coordinates, return immediately after starting the slew. *)
ScopeSlewToCoordinatesAsync[theScope_, ra_Real, dec_Real]:=theScope@SlewToCoordinatesAsync[ra, dec];

(* Move the telescope to the TargetRightAscension and TargetDeclination coordinates, return when slew complete. *)
ScopeSlewToTarget[theScope_]:=theScope@SlewToTarget[];

(* Move the telescope to the TargetRightAscension and TargetDeclination coordinates, returns immediately after starting the slew. *)
ScopeSlewToTargetAsync[theScope_]:=theScope@SlewToTargetAsync[];

(* Matches the scope local horizontal coordinates to the given local horizontal coordinates. *)
ScopeSyncToAltAz[theScope_, az_Real, alt_Real]:=theScope@SyncToAltAz[az, alt];

(* Matches the scope equatorial coordinates to the given equatorial coordinates. *)
ScopeSyncToCoordinates[theScope_, ra_Real, dec_Real]:=theScope@SyncToCoordinates[ra, dec];

(* Matches the scope equatorial coordinates to the given equatorial coordinates. *)
ScopeSyncToTarget[theScope_]:=theScope@SyncToTarget[];

(* Takes telescope out of the Parked state. *)
ScopeUnpark[theScope_]:=theScope@Unpark[];

(* Slew to a named object *)
ScopeSlewToObject[scope_, obj_String] :=
  Module[{ra, dec, rad, decd},
   rad = AstronomicalData[obj, "RightAscension"];
   If[Head[rad] == Missing, Print["Missing RA data for " <> obj]; Return[$Failed]];
   decd = AstronomicalData[obj, "Declination"];
   If[Head[decd] == Missing, Print["Missing Dec data for " <> obj]; Return[$Failed]];
   ra = QuantityMagnitude[UnitConvert[rad, "HoursOfRightAscension"]];
   dec = FromDMS[decd];
   ScopeSlewToCoordinates[scope, ra, dec]
  ];

(* new stuff *)

askDarkFrame=True;

SetAskDarkFrame[ask_]:=Block[{}, askDarkFrame=ask; ask];

(* Set True to connect to the device hardware. Set False to disconnect from the device hardware. You can also read the property to check whether it is connected. This reports the current hardware state. *)
SetScopeConnected[theScope_, val_]:=Block[{}, theScope@Connected = val; theScope@Connected ];

(* The declination tracking rate (arcseconds per SI second, default = 0.0) *)
SetScopeDeclinationRate[theScope_, val_]:=Block[{}, theScope@DeclinationRate = val; theScope@DeclinationRate ];

(* True if the telescope or driver applies atmospheric refraction to coordinates. *)
SetScopeDoesRefraction[theScope_, val_]:=Block[{}, theScope@DoesRefraction = val; theScope@DoesRefraction ];

(* The current Declination movement rate offset for telescope guiding (degrees/sec) *)
SetScopeGuideRateDeclination[theScope_, val_]:=Block[{}, theScope@GuideRateDeclination = val; theScope@GuideRateDeclination ];

(* The current Right Ascension movement rate offset for telescope guiding (degrees/sec) *)
SetScopeGuideRateRightAscension[theScope_, val_]:=Block[{}, theScope@GuideRateRightAscension = val; theScope@GuideRateRightAscension ];

(* The right ascension tracking rate offset from sidereal (seconds per sidereal second, default = 0.0) *)
SetScopeRightAscensionRate[theScope_, val_]:=Block[{}, theScope@RightAscensionRate = val; theScope@RightAscensionRate ];

(* Indicates the pointing state of the mount. *)
SetScopeSideOfPier[theScope_, val_]:=Block[{}, theScope@SideOfPier = val; theScope@SideOfPier ];

(* The elevation above mean sea level (meters) of the site at which the telescope is located *)
SetScopeSiteElevation[theScope_, val_]:=Block[{}, theScope@SiteElevation = val; theScope@SiteElevation ];

(* The geodetic(map) latitude (degrees, positive North, WGS84) of the site at which the telescope is located. *)
SetScopeSiteLatitude[theScope_, val_]:=Block[{}, theScope@SiteLatitude = val; theScope@SiteLatitude ];

(* The longitude (degrees, positive East, WGS84) of the site at which the telescope is located. *)
SetScopeSiteLongitude[theScope_, val_]:=Block[{}, theScope@SiteLongitude = val; theScope@SiteLongitude ];

(* Specifies a post-slew settling time (sec.). *)
SetScopeSlewSettleTime[theScope_, val_]:=Block[{}, theScope@SlewSettleTime = val; theScope@SlewSettleTime ];

(* The declination (degrees, positive North) for the target of an equatorial slew or sync operation *)
SetScopeTargetDeclination[theScope_, val_]:=Block[{}, theScope@TargetDeclination = val; theScope@TargetDeclination ];

(* The right ascension (hours) for the target of an equatorial slew or sync operation *)
SetScopeTargetRightAscension[theScope_, val_]:=Block[{}, theScope@TargetRightAscension = val; theScope@TargetRightAscension ];

(* The state of the telescope's sidereal tracking drive. *)
SetScopeTracking[theScope_, val_]:=Block[{}, theScope@Tracking = val; theScope@Tracking ];

(* The current tracking rate of the telescope's sidereal drive *)
SetScopeTrackingRate[theScope_, val_]:=Block[{}, theScope@TrackingRate = val; theScope@TrackingRate ];

(* The UTC date/time of the telescope's internal clock *)
SetScopeUTCDate[theScope_, val_]:=Block[{}, theScope@UTCDate = val; theScope@UTCDate ];


(* The alignment mode of the mount. *)
GetScopeAlignmentMode[theScope_]:=theScope@AlignmentMode;

(* The Altitude above the local horizon of the telescope's current position (degrees, positive up) *)
GetScopeAltitude[theScope_]:=theScope@Altitude;

(* The area of the telescope's aperture, taking into account any obstructions (square meters) *)
GetScopeApertureArea[theScope_]:=theScope@ApertureArea;

(* The telescope's effective aperture diameter (meters) *)
GetScopeApertureDiameter[theScope_]:=theScope@ApertureDiameter;

(* True if the telescope is stopped in the Home position. Set only following a FindHome() operation, and reset with any slew operation. This property must be False if the telescope does not support homing. *)
GetScopeAtHome[theScope_]:=theScope@AtHome;

(* True if the telescope has been put into the parked state by the see Park() method. Set False by calling the Unpark() method. *)
GetScopeAtPark[theScope_]:=theScope@AtPark;

(* The azimuth at the local horizon of the telescope's current position (degrees, North-referenced, positive East/clockwise). *)
GetScopeAzimuth[theScope_]:=theScope@Azimuth;

(* True if this telescope is capable of programmed finding its home position (FindHome() method). *)
GetScopeCanFindHome[theScope_]:=theScope@CanFindHome;

(* True if this telescope is capable of programmed parking (Park()method) *)
GetScopeCanPark[theScope_]:=theScope@CanPark;

(* True if this telescope is capable of software-pulsed guiding (via the PulseGuide(GuideDirections, Int32) method) *)
GetScopeCanPulseGuide[theScope_]:=theScope@CanPulseGuide;

(* True if the DeclinationRate property can be changed to provide offset tracking in the declination axis. *)
GetScopeCanSetDeclinationRate[theScope_]:=theScope@CanSetDeclinationRate;

(* True if the guide rate properties used for PulseGuide(GuideDirections, Int32) can be adjusted. *)
GetScopeCanSetGuideRates[theScope_]:=theScope@CanSetGuideRates;

(* True if this telescope is capable of programmed setting of its park position (SetPark() method) *)
GetScopeCanSetPark[theScope_]:=theScope@CanSetPark;

(* True if the SideOfPier property can be set, meaning that the mount can be forced to flip. *)
GetScopeCanSetPierSide[theScope_]:=theScope@CanSetPierSide;

(* True if the RightAscensionRate property can be changed to provide offset tracking in the right ascension axis. *)
GetScopeCanSetRightAscensionRate[theScope_]:=theScope@CanSetRightAscensionRate;

(* True if the Tracking property can be changed, turning telescope sidereal tracking on and off. *)
GetScopeCanSetTracking[theScope_]:=theScope@CanSetTracking;

(* True if this telescope is capable of programmed slewing (synchronous or asynchronous) to equatorial coordinates *)
GetScopeCanSlew[theScope_]:=theScope@CanSlew;

(* True if this telescope is capable of programmed slewing (synchronous or asynchronous) to local horizontal coordinates *)
GetScopeCanSlewAltAz[theScope_]:=theScope@CanSlewAltAz;

(* True if this telescope is capable of programmed asynchronous slewing to local horizontal coordinates *)
GetScopeCanSlewAltAzAsync[theScope_]:=theScope@CanSlewAltAzAsync;

(* True if this telescope is capable of programmed asynchronous slewing to equatorial coordinates. *)
GetScopeCanSlewAsync[theScope_]:=theScope@CanSlewAsync;

(* True if this telescope is capable of programmed syncing to equatorial coordinates. *)
GetScopeCanSync[theScope_]:=theScope@CanSync;

(* True if this telescope is capable of programmed syncing to local horizontal coordinates *)
GetScopeCanSyncAltAz[theScope_]:=theScope@CanSyncAltAz;

(* True if this telescope is capable of programmed unparking (Unpark() method). *)
GetScopeCanUnpark[theScope_]:=theScope@CanUnpark;

(* Set True to connect to the device hardware. Set False to disconnect from the device hardware. You can also read the property to check whether it is connected. This reports the current hardware state. *)
GetScopeConnected[theScope_]:=theScope@Connected;

(* The declination (degrees) of the telescope's current equatorial coordinates, in the coordinate system given by the EquatorialSystem property. Reading the property will raise an error if the value is unavailable. *)
GetScopeDeclination[theScope_]:=theScope@Declination;

(* The declination tracking rate (arcseconds per SI second, default = 0.0) *)
GetScopeDeclinationRate[theScope_]:=theScope@DeclinationRate;

(* Returns a description of the device, such as manufacturer and model number. Any ASCII characters may be used. *)
GetScopeDescription[theScope_]:=theScope@Description;

(* True if the telescope or driver applies atmospheric refraction to coordinates. *)
GetScopeDoesRefraction[theScope_]:=theScope@DoesRefraction;

(* Descriptive and version information about this ASCOM driver. *)
GetScopeDriverInfo[theScope_]:=theScope@DriverInfo;

(* A string containing only the major and minor version of the driver. *)
GetScopeDriverVersion[theScope_]:=theScope@DriverVersion;

(* Equatorial coordinate system used by this telescope. *)
GetScopeEquatorialSystem[theScope_]:=theScope@EquatorialSystem;

(* The telescope's focal length, meters *)
GetScopeFocalLength[theScope_]:=theScope@FocalLength;

(* The current Declination movement rate offset for telescope guiding (degrees/sec) *)
GetScopeGuideRateDeclination[theScope_]:=theScope@GuideRateDeclination;

(* The current Right Ascension movement rate offset for telescope guiding (degrees/sec) *)
GetScopeGuideRateRightAscension[theScope_]:=theScope@GuideRateRightAscension;

(* The interface version number that this device supports. *)
GetScopeInterfaceVersion[theScope_]:=theScope@InterfaceVersion;

(* True if a PulseGuide(GuideDirections, Int32) command is in progress, False otherwise *)
GetScopeIsPulseGuiding[theScope_]:=theScope@IsPulseGuiding;

(* The short name of the driver, for display purposes *)
GetScopeName[theScope_]:=theScope@Name;

(* The right ascension (hours) of the telescope's current equatorial coordinates, in the coordinate system given by the EquatorialSystem property *)
GetScopeRightAscension[theScope_]:=theScope@RightAscension;

(* The right ascension tracking rate offset from sidereal (seconds per sidereal second, default = 0.0) *)
GetScopeRightAscensionRate[theScope_]:=theScope@RightAscensionRate;

(* Indicates the pointing state of the mount. *)
GetScopeSideOfPier[theScope_]:=theScope@SideOfPier;

(* The local apparent sidereal time from the telescope's internal clock (hours, sidereal) *)
GetScopeSiderealTime[theScope_]:=theScope@SiderealTime;

(* The elevation above mean sea level (meters) of the site at which the telescope is located *)
GetScopeSiteElevation[theScope_]:=theScope@SiteElevation;

(* The geodetic(map) latitude (degrees, positive North, WGS84) of the site at which the telescope is located. *)
GetScopeSiteLatitude[theScope_]:=theScope@SiteLatitude;

(* The longitude (degrees, positive East, WGS84) of the site at which the telescope is located. *)
GetScopeSiteLongitude[theScope_]:=theScope@SiteLongitude;

(* True if telescope is currently moving in response to one of the Slew methods or the MoveAxis(TelescopeAxes, Double) method, False at all other times. *)
GetScopeSlewing[theScope_]:=theScope@Slewing;

(* Specifies a post-slew settling time (sec.). *)
GetScopeSlewSettleTime[theScope_]:=theScope@SlewSettleTime;

(* Returns the list of custom action names supported by this driver. *)
GetScopeSupportedActions[theScope_]:=theScope@SupportedActions;

(* The declination (degrees, positive North) for the target of an equatorial slew or sync operation *)
GetScopeTargetDeclination[theScope_]:=theScope@TargetDeclination;

(* The right ascension (hours) for the target of an equatorial slew or sync operation *)
GetScopeTargetRightAscension[theScope_]:=theScope@TargetRightAscension;

(* The state of the telescope's sidereal tracking drive. *)
GetScopeTracking[theScope_]:=theScope@Tracking;

(* The current tracking rate of the telescope's sidereal drive *)
GetScopeTrackingRate[theScope_]:=theScope@TrackingRate;

(* Returns a collection of supported DriveRates values that describe the permissible values of the TrackingRate property for this telescope type. *)
GetScopeTrackingRates[theScope_]:=theScope@TrackingRates;

(* The UTC date/time of the telescope's internal clock *)
GetScopeUTCDate[theScope_]:=theScope@UTCDate;


(* Dome Functions *)
(*
  From Ascom-Standards.org:
  This interface can be used to control most types of observatory structure, including domes (with or without a controllable shutter), clamshells and roll off roofs.

  Dome Coordinates

  The azimuth and shutter altitude coordinates within this interface refer to positioning of the dome itself and are not sky coordinates from the perspective of the observing device. Mount geometry and pier location often mean that the observing device is not located at the centre point of a hemispherical dome and this results in the dome needing to be positioned at different coordinates than those of the mount in order to expose the required part of the sky to the observing device.

  Calculating the required dome position is not the responsibility of the dome driver, this must be done by the client application or by an intermediary hub such as the Device Hub. The coordinates sent to the dome driver must be those required to position the dome slit in the correct position after allowing for the geometry of the mount.

  Dome Slaving

  A dome is said to be slaved when its azimuth and shutter altitude are controlled by a "behind the scenes" controller that knows the required telescope observing coordinates and that can calculate the required dome position after allowing for mount geometry, observing device orientation etc. When slaved and in operational use most of the dome interface methods are of little relevance apart from the shutter control methods.

  How to use the Dome interface to implement a Roll-off Roof or Clamshell

  A roll off roof or clamshell is implemented using the shutter control as the roof. The properties and methods should be implemented as follows:

  OpenShutter() and CloseShutter() open and close the roof
  CanFindHome, CanPark, CanSetAltitude, CanSetAzimuth, CanSetPark, CanSlave and CanSyncAzimuth all return false.
  CanSetShutter returns true.
  ShutterStatus should be implemented.
  AbortSlew() should stop the roof or shutter.
  FindHome(), Park(), SetPark(), SlewToAltitude(Double), SlewToAzimuth(Double) and SyncToAzimuth(Double) all throw MethodNotImplementedException
  Altitude and Azimuth throw PropertyNotImplementedException
*)

DomeAltitude[theDome_]:=theDome@Altitude;
DomeAtHome[theDome_]:=theDome@AtHome;
DomeAtPark[theDome_]:=theDome@AtPark;
DomeAzimuth[theDome_]:=theDome@Azimuth;
DomeCanFindHome[theDome_]:=theDome@CanFindHome;
DomeCanPark[theDome_]:=theDome@CanPark;
DomeCanSetAltitude[theDome_]:=theDome@CanSetAltitude;
DomeCanSetAzimuth[theDome_]:=theDome@CanSetAzimuth;
DomeCanSetPark[theDome_]:=theDome@CanSetPark;
DomeCanSetShutter[theDome_]:=theDome@CanSetShutter;
DomeCanSlave[theDome_]:=theDome@CanSlave;
DomeCanSyncAzimuth[theDome_]:=theDome@CanSyncAzimuth;
DomeConnected[theDome_]:=theDome@Connected;
DomeDescription[theDome_]:=theDome@Description;
DomeDriverInfo[theDome_]:=theDome@DriverInfo;
DomeDriverVersion[theDome_]:=theDome@DriverVersion;
DomeInterfaceVersion[theDome_]:=theDome@InterfaceVersion;
DomeName[theDome_]:=theDome@Name;
DomeShutterStatus[theDome_]:=theDome@ShutterStatus;
DomeSlaved[theDome_]:=theDome@Slaved;
DomeSlewing[theDome_]:=theDome@Slewing;
DomeSupportedActions[theDome_]:=theDome@SupportedActions;

DomeAbortSlew[theDome_]:=theDome@AbortSlew[];
DomeAction[theDome_,actionName_String, actionParameters_String]:=theDome@Action[actionName, actionParameters];
(* is this a static method
DomeChoose[theDome_,]:=
*)
DomeCloseShutter[theDome_]:=theDome@CloseShutter[];
DomeCommandBlind[theDome_, theCommand_String, isRaw_]:= theDome@CommandBlind[theCommand, isRaw];
DomeCommandBool[theDome_, theCommand_String, isRaw_]:= theDome@CommandBool[theCommand, isRaw];
DomeCommandString[theDome_, theCommand_String, isRaw_]:= theDome@CommandString[theCommand, isRaw];
DomeDispose[theDome_]:=theDome@Dispose[];
DomeFindHome[theDome_]:=theDome@FindHome[];
DomeOpenShutter[theDome_]:=theDome@OpenShutter[];
DomePark[theDome_]:=theDome@Park[];
DomeSetPark[theDome_]:=theDome@SetPark[];
DomeSetupDialog[theDome_]:=theDome@SetupDialog[];
DomeSlewToAltitude[theDome_, theAltitude_Real]:=theDome@SlewToAltitude[theAltitude];
DomeSlewToAzimuth[theDome_, theAzimuth_Real]:=theDome@SlewToAzimuth[theAzimuth];
DomeSyncToAzimuth[theDome_, theAzimuth_Real]:=theDome@SyncToAzimuth[theAzimuth];


(* Focuser Functions *)
(*
  Stops focuser movement
*)
HaltFocuser[theFocuser_]:=theFocuser@Halt[];

(*
  Moves the focuser by the specified amount or to the specified position depending on the value of the Absolute property. 
*)
MoveFocuser[theFocuser_, posOrAmount_]:=theFocuser@Move[posOrAmount];

(*
  Returns True if the focuser is capable of absolute position; that is, being commanded to a specific step location. 
*)
IsFocuserAbsolute[theFocuser_]:=theFocuser@Absolute;

(*
  Maximum increment size allowed by the focuser; i.e. the maximum number of steps allowed in one move operation. 
*)
GetFocuserMaxIncrement[theFocuser_]:=theFocuser@MaxIncrement;

(*
  Maximum step position permitted. 
*)
GetFocuserMaxStep[theFocuser_]:=theFocuser@MaxStep;

(*
  Current focuser position, in steps. 
*)
GetFocuserPosition[theFocuser_]:=theFocuser@Position;

(*
  Step size (microns) for the focuser. 
*)
GetFocuserStepSize[theFocuser_]:=theFocuser@StepSize;

(* Camera Functions *)

GetCameraBinX[theCamera_]:=theCamera@BinX;
GetCameraBinY[theCamera_]:=theCamera@BinY;
GetCameraConnected[theCamera_]:=theCamera@Connected;
GetCameraCoolerOn[theCamera_]:=theCamera@CoolerOn;
GetCameraFastReadout[theCamera_]:=theCamera@FastReadout;
GetCameraGain[theCamera_]:=theCamera@Gain;
GetCameraNumX[theCamera_]:=theCamera@NumX;
GetCameraNumY[theCamera_]:=theCamera@NumY;
GetCameraOffset[theCamera_]:=theCamera@Offset;
GetCameraReadoutMode[theCamera_]:=theCamera@ReadoutMode;
GetCameraCCDTemperature[theCamera_]:=theCamera@SetCCDTemperature;
GetCameraStartX[theCamera_]:=theCamera@StartX;
GetCameraStartY[theCamera_]:=theCamera@StartY;
GetCameraSubExposureDuration[theCamera_]:=theCamera@SubExposureDuration;

SetCameraBinX[theCamera_, theValue_]:=Block[{},theCamera@BinX = theValue; theCamera@BinX ];
SetCameraBinY[theCamera_, theValue_]:=Block[{},theCamera@BinY = theValue; theCamera@BinY ];
SetCameraConnected[theCamera_, theValue_]:=Block[{},theCamera@Connected = theValue; theCamera@Connected ];
SetCameraCoolerOn[theCamera_, theValue_]:=Block[{},theCamera@CoolerOn = theValue; theCamera@CoolerOn ];
SetCameraFastReadout[theCamera_, theValue_]:=Block[{},theCamera@FastReadout = theValue; theCamera@FastReadout ];
SetCameraGain[theCamera_, theValue_]:=Block[{},theCamera@Gain = theValue; theCamera@Gain ];
SetCameraNumX[theCamera_, theValue_]:=Block[{},theCamera@NumX = theValue; theCamera@NumX ];
SetCameraNumY[theCamera_, theValue_]:=Block[{},theCamera@NumY = theValue; theCamera@NumY ];
SetCameraOffset[theCamera_, theValue_]:=Block[{},theCamera@Offset = theValue; theCamera@Offset ];
SetCameraReadoutMode[theCamera_, theValue_]:=Block[{},theCamera@ReadoutMode = theValue; theCamera@ReadoutMode ];
SetCameraCCDTemperature[theCamera_, theValue_]:=Block[{},theCamera@SetCCDTemperature = theValue; theCamera@SetCCDTemperature ];
SetCameraStartX[theCamera_, theValue_]:=Block[{},theCamera@StartX = theValue; theCamera@StartX ];
SetCameraStartY[theCamera_, theValue_]:=Block[{},theCamera@StartY = theValue; theCamera@StartY ];
SetCameraSubExposureDuration[theCamera_, theValue_]:=Block[{},theCamera@SubExposureDuration = theValue; theCamera@SubExposureDuration ];

GetCameraBayerOffsetX[theCamera_]:=theCamera@BayerOffsetX;
GetCameraBayerOffsetY[theCamera_]:=theCamera@BayerOffsetY;
GetCameraState[theCamera_]:=theCamera@CameraState;
GetCameraXSize[theCamera_]:=theCamera@CameraXSize;
GetCameraYSize[theCamera_]:=theCamera@CameraYSize;
GetCameraCanAbortExposure[theCamera_]:=theCamera@CanAbortExposure;
GetCameraCanAsymmetricBin[theCamera_]:=theCamera@CanAsymmetricBin;
GetCameraCanFastReadout[theCamera_]:=theCamera@CanFastReadout;
GetCameraCanGetCoolerPower[theCamera_]:=theCamera@CanGetCoolerPower;
GetCameraCanPulseGuide[theCamera_]:=theCamera@CanPulseGuide;
GetCameraCanSetCCDTemperature[theCamera_]:=theCamera@CanSetCCDTemperature;
GetCameraCanStopExposure[theCamera_]:=theCamera@CanStopExposure;
GetCameraCCDTemperature[theCamera_]:=theCamera@CCDTemperature;
GetCameraCoolerPower[theCamera_]:=theCamera@CoolerPower;
GetCameraDescription[theCamera_]:=theCamera@Description;
GetCameraDriverInfo[theCamera_]:=theCamera@DriverInfo;
GetCameraDriverVersion[theCamera_]:=theCamera@DriverVersion;
GetCameraElectronsPerADU[theCamera_]:=theCamera@ElectronsPerADU;
GetCameraExposureMax[theCamera_]:=theCamera@ExposureMax;
GetCameraExposureMin[theCamera_]:=theCamera@ExposureMin;
GetCameraExposureResolution[theCamera_]:=theCamera@ExposureResolution;
GetCameraFullWellCapacity[theCamera_]:=theCamera@FullWellCapacity;
GetCameraGainMax[theCamera_]:=theCamera@GainMax;
GetCameraGainMin[theCamera_]:=theCamera@GainMin;
GetCameraGains[theCamera_]:=theCamera@Gains;
GetCameraHasShutter[theCamera_]:=theCamera@HasShutter;
GetCameraHeatSinkTemperature[theCamera_]:=theCamera@HeatSinkTemperature;
GetCameraImageArray[theCamera_]:=theCamera@ImageArray;
GetCameraImageArrayVariant[theCamera_]:=theCamera@ImageArrayVariant;
GetCameraImageReady[theCamera_]:=theCamera@ImageReady;
GetCameraInterfaceVersion[theCamera_]:=theCamera@InterfaceVersion;
GetCameraIsPulseGuiding[theCamera_]:=theCamera@IsPulseGuiding;
GetCameraLastExposureDuration[theCamera_]:=theCamera@LastExposureDuration;
GetCameraLastExposureStartTime[theCamera_]:=theCamera@LastExposureStartTime;
GetCameraMaxADU[theCamera_]:=theCamera@MaxADU;
GetCameraMaxBinX[theCamera_]:=theCamera@MaxBinX;
GetCameraMaxBinY[theCamera_]:=theCamera@MaxBinY;
GetCameraName[theCamera_]:=theCamera@Name;
GetCameraOffsetMax[theCamera_]:=theCamera@OffsetMax;
GetCameraOffsetMin[theCamera_]:=theCamera@OffsetMin;
GetCameraOffsets[theCamera_]:=theCamera@Offsets;
GetCameraPercentCompleted[theCamera_]:=theCamera@PercentCompleted;
GetCameraPixelSizeX[theCamera_]:=theCamera@PixelSizeX;
GetCameraPixelSizeY[theCamera_]:=theCamera@PixelSizeY;
GetCameraReadoutModes[theCamera_]:=theCamera@ReadoutModes;
GetCameraSensorName[theCamera_]:=theCamera@SensorName;
GetCameraSensorType[theCamera_]:=theCamera@SensorType;
GetCameraSupportedActions[theCamera_]:=theCamera@SupportedActions;

ActivateCameraCooler[theCamera_, coolerOn_]:=
Module[{}
  theCamera@CoolerOn = coolerOn;
];

GetCameraSensorTemperature[theCamera_]:=
Module[{},
  theCamera@CCDTemperature
];

GetCameraAmbientTemperature[theCamera_]:=
Module[{},
  theCamera@HeatSinkTemperature
];

GetCameraCoolerPowerLevel[theCamera_]:=
Module[{},
  theCamera@CoolerPower
];

SetCameraSensorTemperature[theCamera_, tempDegreesCelcius_]:=
Module[{},
  theCamera@SetCCDTemperature = tempDegreesCelcius;
  tempDegreesCelcius
];

SetCameraBinning[theCamera_, binningXY_List]:=
Module[{binx,biny},
  binx = binningXY[[1]];
  biny = binningXY[[2]];

  If[binx > GetCameraMaxBinX[theCamera], Return[$Failed]];
  If[biny > GetCameraMaxBinY[theCamera], Return[$Failed]];
  If[binx < 1, Return[$Failed]];
  If[biny < 1, Return[$Failed]];
  xres = GetCameraXSize[theCamera]/binx;
  yres = GetCameraYSize[theCamera]/biny;
  theCamera@BinX = binx;
  theCamera@BinY = biny;
  SetCameraNumX[theCamera, xres];
  SetCameraNumY[theCamera, yres];
  binningXY
];

SetCameraBinning[theCamera_, bin_Integer]:=
Module[{xres, yres},
  If[bin > GetCameraMaxBinX[theCamera], Return[$Failed]];
  If[bin > GetCameraMaxBinY[theCamera], Return[$Failed]];
  If[bin < 1, Return[$Failed]];
  xres = GetCameraXSize[theCamera]/bin;
  yres = GetCameraYSize[theCamera]/bin;
  theCamera@BinX = bin;
  theCamera@BinY = bin;
  SetCameraNumX[theCamera, xres];
  SetCameraNumY[theCamera, yres];
  {bin, bin}
];

GetCameraBinning[theCamera_]:=
Module[{},
  List[theCamera@BinX, theCamera@BinY]
];

GetCameraDimensions[theCamera_]:=
Module[{},
  List[theCamera@CameraXSize, theCamera@CameraYSize]
];

GetSubframeDimensions[theCamera_]:=
Module[{},
  List[List[theCamera@StartX, theCamera@StartY], List[theCamera@NumX, theCamera@NumY]]
];

SetSubframeDimensions[theCamera_, subframeDimensions_List]:=
Module[{},
  theCamera@StartX  = subframeDimensions[[1]][[1]];
  theCamera@StartY  = subframeDimensions[[1]][[2]];
  theCamera@NumX  = subframeDimensions[[2]][[1]];
  theCamera@NumY  = subframeDimensions[[2]][[2]];
  subframeDimensions
];

GetPixelDimensionsMicrons[theCamera_]:=
Module[{},
  List[theCamera@PixelSizeX, theCamera@PixelSizeY]
];

(*
  Monochrome  Camera produces monochrome array with no Bayer encoding
  Colour      Camera produces color image directly, requiring not Bayer decoding
  RGGB        Camera produces RGGB encoded Bayer array images
  CMYG        Camera produces CMYG encoded Bayer array images
  CMYG2       Camera produces CMYG2 encoded Bayer array images
  LRGB        Camera produces Kodak TRUESENSE Bayer LRGB array images
*)
GetSensorType[theCamera_]:=
Module[{type, typenames = {"Monochrome", "Color", "RGGB", "CMYG", "CMYG2", "LRGB"}},
  type = theCamera@SensorType;
  If[type < 0 || type > 5, Return["Unknown"]];
  typenames[[type + 1]]
];

GetBayerOffset[theCamera_]:=
Module[{},
  List[theCamera@BayerOffsetX, theCamera@BayerOffsetY]
];

AcquireImage[theCamera_, duration_, shutterOpen_] :=
 Module[{},
  theCamera@StartExposure[duration, shutterOpen];
  While[theCamera@ImageReady != True, Pause[0.5]];
  Image[Transpose[theCamera@ImageArray], "Bit16"]
];

AcquireImageToFile[theCamera_, duration_, shutterOpen_, fileName_] :=
Module[{im},
  theCamera@StartExposure[duration, shutterOpen];
  While[theCamera@ImageReady != True, Pause[0.5]];
  im = Image[Transpose[theCamera@ImageArray], "Bit16"];
  Export[fileName, ImageData[im, Interleaving -> False, DataReversed -> True]]
];

AcquireImageToFITSFile[theCamera_, duration_, shutterOpen_, fileName_] :=
Module[{im, sample},
  theCamera@StartExposure[duration, shutterOpen];
  While[theCamera@ImageReady != True, Pause[0.5]];
  im = Image[Transpose[theCamera@ImageArray], "Bit16"];
(*
  sample=Import["ExampleData/sample.fits"]; 
  meta=Import[f,"MetaInformation"]; 
  meta = AssociateTo[meta[1, "GENERAL INFORMATION"], "CTAG1" -> DateString[Now]]; 
  

  Export[fileName, ImageData[im, Interleaving -> False, DataReversed -> True], "MetaInformation"->meta]
*)
  Export[fileName, ImageData[im, Interleaving -> False, DataReversed -> True]]

];

buildImageFITSFileName[prefix_String, outputDir_String, index_] :=
 FileNameJoin[{outputDir, 
   prefix <> "_" <> 
    StringReplace[DateString["DateShort"], " " -> "_"] <> "_" <> 
    ToString[index] <> ".fit"}];

AcquireFITSImageSequence[theCamera_, duration_, shutterOpen_, count_, prefix_String, outputDir_String] :=
Module[{i, fileName, imageType},

  If[shutterOpen == False && theCamera@HasShutter == False && askDarkFrame == True, 
    CreateDialog[{TextCell["Acquiring dark frames.  Cover the telescope objective"], DefaultButton[]}]
  ];

  For[i = 1, i <= count, i++, 
    fileName = buildImageFITSFileName[prefix, outputDir, i];
    (*Print["Acquiring image: " <> ToString[i] <> " of " <> ToString[count] <> ": " <> fileName];*)
    AcquireImageToFITSFile[theCamera, duration, shutterOpen, fileName]
  ];

  If[shutterOpen == False && theCamera@HasShutter == False && askDarkFrame == True, 
   CreateDialog[{TextCell["Acquisition complete.  Uncover the telescope objective"], DefaultButton[]}]
  ];
];

(* Filter Wheel Functions*)
SetFilterWheelPosition[theFilterWheel_, filterPos_]:=
Module[{},
  theFilterWheel@Position = filterPos - 1;
  filterPos
];

SetFilterWheelPosition[theFilterWheel_, filterName_String]:=
Module[{l},
  l = FirstPosition[GetFilterWheelNames[theFilterWheel], filterName, {}];
  If[Length[l]==1, 
    SetFilterWheelPosition[theFilterWheel, First[l]]; filterName,
    $Failed
    ]
];

GetFilterWheelPosition[theFilterWheel_]:= theFilterWheel@Position + 1;

GetFilterWheelPosition[theFilterWheel_, filterName_String]:=
Module[{l},
  l = FirstPosition[GetFilterWheelNames[theFilterWheel], filterName, {}];
  If[Length[l]==1, 
    First[l],
    $Failed
    ]
];

(*
  Get the name of the current filter
*)
GetFilterWheelCurrentName[theFilterWheel_]:=
  GetFilterWheelNames[theFilterWheel][[GetFilterWheelPosition[theFilterWheel]]];

GetFilterWheelFocusOffsets[theFilterWheel_]:=
Module[{},
  theFilterWheel@FocusOffsets
];

GetFilterWheelFocusOffset[theFilterWheel_, theFilterPos_]:=
Module[{focusOffsets},

  focusOffsets  = GetFilterWheelFocusOffsets[theFilterWheel];

  If[theFilterPos >= 1 && theFilterPos <= Length[focusOffsets],
    focusOffsets[[theFilterPos]],
    $Failed
  ]
];

GetFilterWheelFocusOffset[theFilterWheel_, theFilterName_String]:=
Module[{theFilterPos},
  theFilterPos = GetFilterWheelPosition[theFilterWheel, theFilterName];
  GetFilterWheelFocusOffset[theFilterWheel, theFilterPos]
];

GetFilterWheelNames[theFilterWheel_]:=
Module[{},
  theFilterWheel@Names
];

(* Imaging utility functions *)

(* take count dark frame exposures and store their FITS files in the outputDir *)
AcquireDarkFrameSequence[theCamera_, durationSec_, count_, prefix_String, outputDir_String] :=
Module[{sensorTemp},
	sensorTemp = ToString[Round@GetCameraSensorTemperature[theCamera]]<>"DegreesC";
	Print["Acquiring dark frames."];
  AcquireFITSImageSequence[theCamera, durationSec, False, count, prefix <> "_" <> sensorTemp <> "_" <> ToString[durationSec] <> "Sec_Dark", outputDir];
]; 

(* take count bias frame exposures and store their FITS files in the outputDir *)
AcquireBiasFrameSequence[theCamera_, count_, prefix_String, outputDir_String] :=
Module[{exposureMin},
  exposureMin = GetCameraExposureMin[theCamera];
	Print["Acquiring bias frames."];
  AcquireFITSImageSequence[theCamera, exposureMin, False, count, prefix <> "_Bias", outputDir];
]; 

(* 
  Make a plot of the median intensity of a flat frame as a function of time ranging from 0 to maxTime in seconds.
  This is used to identify the linear region where flat frames should be exposed
*)
PlotFlatFrameResponse[theCamera_, maxTimeSeconds_, count_]:=
  ListPlot[{#, Median@Flatten@ImageData@AcquireImage[theCamera, #, True]} & /@ Range[0.0, maxTimeSeconds, maxTimeSeconds/count]];

(* Determine best flat frame exposure time in the range 0.0 to maxTimeSeconds *)
DetermineOptimalFlatExposure[theCamera_, maxTimeSeconds_] :=
Module[{data, line, result, count = 10},
  data = {#, Median@Flatten@ImageData@AcquireImage[theCamera, #, True]}&/@Range[0.0, maxTimeSeconds, maxTimeSeconds / count];
  line = Fit[data, {1, x}, x];
  result = Solve[line == y, x];
  (* In the linear region, return the time that results in about 45% of the max ADU for the optimal flat time*)
  result[[1, 1, 2]] /. y -> 0.45
];

(* take count flat frame exposures and store their FITS files in the outputDir *)
AcquireFlatFrameSequence[theCamera_, count_, exposureTime_, prefix_String, outputDir_String] :=
Module[{},
	Print["Acquiring flat frames."];
  AcquireFITSImageSequence[theCamera, exposureTime, True, count, prefix <> "_Flat", outputDir];
]; 

AcquireRGBSequence[theCamera_, theFilterWheel_, frameExposure_, frameCount_, binning_, outputDir_] :=
Module[{currentFilterPos, currentBinning},
  currentBinning = GetCameraBinning[theCamera];
  currentFilterPos = GetFilterWheelPosition[theFilterWheel];
  SetCameraBinning[theCamera, binning];
  Print["Acquiring Red Frames..."];
  SetFilterWheelPosition[theFilterWheel, "Red"];
  AcquireFITSImageSequence[theCamera, frameExposure, True, frameCount, "red", outputDir];
  Print["Acquiring Green Frames..."]; 
  SetFilterWheelPosition[theFilterWheel, "Green"];
  AcquireFITSImageSequence[theCamera, frameExposure, True, frameCount, "green", outputDir];
  Print["Acquiring Blue Frames..."];
  SetFilterWheelPosition[theFilterWheel, "Blue"];
  AcquireFITSImageSequence[theCamera, frameExposure, True, frameCount, "blue", outputDir];
  SetCameraBinning[theCamera, currentBinning];
  SetFilterWheelPos[theFilterWheel, currentFilterPos];
];

AcquireLRGBSequence[theCamera_, theFilterWheel_, frameExposure_, frameCount_, outputDir_] :=
Module[{currentFilterPos, currentBinning},
  currentBinning = GetCameraBinning[theCamera];
  currentFilterPos = GetFilterWheelPosition[theFilterWheel];
  (* Use low resolution for RGB frames *)
  AcquireRGBSequence[theCamera, theFilterWheel, frameExposure, frameCount, 2, outputDir];
  Print["Acquiring Luminance Frames..."];
  (* Use full resolution for luminance frames *)
  SetCameraBinning[theCamera, 1];
  SetFilterWheelPosition[theFilterWheel, "Luminance"];
  AcquireFITSImageSequence[theCamera, frameExposure, True, frameCount, "luminance", outputDir];
  SetCameraBinning[theCamera, currentBinning];
  SetFilterWheelPos[theFilterWheel, currentFilterPos];
];

AcquireDarkBiasFrames[camera_, filterwheel_, exposure_, count_, outputDir_String] :=
Module[{biasDir, darkDir, currentFilterPos, currentBinning},

  currentBinning = GetCameraBinning[theCamera];
  currentFilterPos = GetFilterWheelPosition[theFilterWheel];

  Print["Acquiring " <> ToString[count] <> " bias frames"];
  biasDir = FileNameJoin[{outputDir, "Bias"}];
  SetFilterWheelPosition[filterwheel, "Dark Frame"];
  
  SetCameraBinning[camera, 1]; 
  AcquireFITSImageSequence[camera, GetCameraExposureMin[camera], False, count, "biashi", biasDir]; 
  
  SetCameraBinning[camera, 2]; 
  AcquireFITSImageSequence[camera, GetCameraExposureMin[camera], False, count, "biaslo", biasDir];
  
  Print["Acquiring " <> ToString[count] <> " dark frames"];
  darkDir = FileNameJoin[{outputDir, "Darks"}];
  SetFilterWheelPosition[filterwheel, "Dark Frame"];
  SetCameraBinning[camera, 1]; 
  AcquireFITSImageSequence[camera, exposure, False, count, "darkhi", darkDir]; SetCameraBinning[camera, 2]; 
  AcquireFITSImageSequence[camera, exposure, False, count, "darklo", darkDir];

  SetCameraBinning[theCamera, currentBinning];
  SetFilterWheelPos[theFilterWheel, currentFilterPos];

];

(* Deprecated
AcquireImageSequence[theCamera_, durationSec_, shutterOpen_, count_, prefix_String, outputDir_String] :=
  AcquireFITSImageSequence[theCamera, durationSec, shutterOpen, count, prefix, outputDir];

AcquireFilteredImageSequence[theCamera_, theFilterWheel_, theFocuser_, durationSec_, shutterOpen_, count_, prefix_String, outputDir_String, filter_String] :=
Module[{filterOffset},
  filterOffset = GetFilterWheelFocusOffset[theFilterWheel, filter];
  MoveFocuser[theFocuser, filterOffset];
  SetFilterWheelPosition[theFilterWheel, filter];
  AcquireFITSImageSequence[theCamera, durationSec, shutterOpen, count, prefix <> "_" <> filter, outputDir];
]; 

AcquireFilteredImageSequence[theCamera_, theFilterWheel_, durationSec_, shutterOpen_, count_, prefix_String, outputDir_String, filter_String] :=
Module[{},
  SetFilterWheelPosition[theFilterWheel, filter];
  AcquireFITSImageSequence[theCamera, durationSec, shutterOpen, count, prefix <> "_" <> filter, outputDir];
]; 


AcquireFilteredCalibrationSet[theCamera_, theFilterWheel_, theFocuser_, exposure_, flatexposure_, biasexposure_, frameCount_, objectName_String, outputDir_String, filterSet_List]:=
Module[{sensorTemp},
	sensorTemp = ToString[GetCameraSensorTemperature[theCamera]]<>"DegreesC";
	Print["Calibration Phase 1 of 4.  Acquiring light flat frames for each filter."];
	AcquireFilteredImageSequence[theCamera,theFilterWheel,theFocuser,flatexposure,True,frameCount,objectName<>"LightFlat",outputDir,#]&/@filterSet;
	Print["Calibration Phase 2 of 4.  Acquiring dark flat frames."];
	AcquireFilteredImageSequence[theCamera,theFilterWheel,theFocuser,flatexposure,False,frameCount,objectName<>"DarkFlat",outputDir,sensorTemp];
	Print["Calibration Phase 3 of 4.  Acquiring bias frames."];
	AcquireFilteredImageSequence[theCamera,theFilterWheel,theFocuser,biasexposure,False,frameCount,objectName<>"Bias",outputDir,"Clear"];
	Print["Calibration Phase 4 of 4.  Acquiring dark frames."];
	AcquireFilteredImageSequence[theCamera,theFilterWheel,theFocuser,exposure,False,frameCount,objectName<>"Dark",outputDir,sensorTemp];
	Print["Calibration sequence complete."];
];

AcquireFilteredImageSet[theCamera_, theFilterWheel_, theFocuser_, exposure_, frameCount_, objectName_String , outputDir_String, filterSet_List]:=
Module[{},
	Print["Acquiring imaging frames for each filter in the set."];
	AcquireFilteredImageSequence[theCamera, theFilterWheel, theFocuser, exposure, True, frameCount, objectName <> "Light",outputDir,#]&/@filterSet;
	Print["Imaging sequence complete."];
];

*)



End[]
EndPackage[]
